@@ -142,26 +142,13 @@ namespace Ogre
 
     void GpuProgram::loadImpl(void)
     {
+        if(mCompileError)
+            return;
+
         // Call polymorphic load
         try 
         {
             loadFromSource();
-
-            if (mDefaultParams)
-            {
-                // Keep a reference to old ones to copy
-                GpuProgramParametersSharedPtr savedParams = mDefaultParams;
-                // reset params to stop them being referenced in the next create
-                mDefaultParams.reset();
-
-                // Create new params
-                mDefaultParams = createParameters();
-
-                // Copy old (matching) values across
-                // Don't use copyConstantsFrom since program may be different
-                mDefaultParams->copyMatchingNamedConstantsFrom(*savedParams.get());
-
-            }
         }
         catch (const RuntimeAssertionException&)
         {
@@ -176,6 +163,24 @@ namespace Ogre
             mCompileError = true;
         }
     }
+    void GpuProgram::postLoadImpl()
+    {
+        if (!mDefaultParams || mCompileError)
+            return;
+
+        // Keep a reference to old ones to copy
+        GpuProgramParametersSharedPtr savedParams = mDefaultParams;
+        // reset params to stop them being referenced in the next create
+        mDefaultParams.reset();
+
+        // Create new params
+        mDefaultParams = createParameters();
+
+        // Copy old (matching) values across
+        // Don't use copyConstantsFrom since program may be different
+        mDefaultParams->copyMatchingNamedConstantsFrom(*savedParams.get());
+    }
+
     //-----------------------------------------------------------------------------
     bool GpuProgram::isRequiredCapabilitiesSupported(void) const
     {
@@ -183,9 +188,7 @@ namespace Ogre
             Root::getSingleton().getRenderSystem()->getCapabilities();
 
         // Basic support check
-        if ((getType() == GPT_VERTEX_PROGRAM && !caps->hasCapability(RSC_VERTEX_PROGRAM)) ||
-            (getType() == GPT_GEOMETRY_PROGRAM && !caps->hasCapability(RSC_GEOMETRY_PROGRAM)) ||
-            (getType() == GPT_FRAGMENT_PROGRAM && !caps->hasCapability(RSC_FRAGMENT_PROGRAM)) ||
+        if ((getType() == GPT_GEOMETRY_PROGRAM && !caps->hasCapability(RSC_GEOMETRY_PROGRAM)) ||
             (getType() == GPT_DOMAIN_PROGRAM && !caps->hasCapability(RSC_TESSELLATION_DOMAIN_PROGRAM)) ||
             (getType() == GPT_HULL_PROGRAM && !caps->hasCapability(RSC_TESSELLATION_HULL_PROGRAM)) ||
             (getType() == GPT_COMPUTE_PROGRAM && !caps->hasCapability(RSC_COMPUTE_PROGRAM)))
