@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,9 +27,6 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 #include "OgrePixelFormat.h"
-#include "OgreBitwise.h"
-#include "OgreColourValue.h"
-#include "OgreException.h"
 #include "OgrePixelFormatDescriptions.h"
 
 namespace {
@@ -39,44 +36,49 @@ namespace {
 namespace Ogre {
 
     //-----------------------------------------------------------------------
-	size_t PixelBox::getConsecutiveSize() const
-	{
-		return PixelUtil::getMemorySize(getWidth(), getHeight(), getDepth(), format);
-	}
-	PixelBox PixelBox::getSubVolume(const Box &def) const
-	{
-		if(PixelUtil::isCompressed(format))
-		{
-			if(def.left == left && def.top == top && def.front == front &&
-			   def.right == right && def.bottom == bottom && def.back == back)
-			{
-				// Entire buffer is being queried
-				return *this;
-			}
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Cannot return subvolume of compressed PixelBuffer", "PixelBox::getSubVolume");
-		}
-		if(!contains(def))
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Bounds out of range", "PixelBox::getSubVolume");
-
-		const size_t elemSize = PixelUtil::getNumElemBytes(format);
-		// Calculate new data origin
-		// Notice how we do not propagate left/top/front from the incoming box, since
-		// the returned pointer is already offset
-		PixelBox rval(def.getWidth(), def.getHeight(), def.getDepth(), format, 
-			((uint8*)data) + ((def.left-left)*elemSize)
-			+ ((def.top-top)*rowPitch*elemSize)
-			+ ((def.front-front)*slicePitch*elemSize)
-		);
-
-		rval.rowPitch = rowPitch;
-		rval.slicePitch = slicePitch;
-		rval.format = format;
-
-		return rval;
-	}
-    void* PixelBox::getTopLeftFrontPixelPtr() const
+    size_t PixelBox::getConsecutiveSize() const
     {
-        return (uint8*)data + (left + top * rowPitch + front * slicePitch) * PixelUtil::getNumElemBytes(format);
+        return PixelUtil::getMemorySize(getWidth(), getHeight(), getDepth(), format);
+    }
+    PixelBox PixelBox::getSubVolume(const Box &def, bool resetOrigin /* = true */) const
+    {
+        if(!contains(def))
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Bounds out of range", "PixelBox::getSubVolume");
+
+        if(PixelUtil::isCompressed(format) && (def.left != left || def.top != top || def.right != right || def.bottom != bottom))
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Cannot return subvolume of compressed PixelBuffer with less than slice granularity", "PixelBox::getSubVolume");
+
+        // Calculate new pixelbox and optionally reset origin.
+        PixelBox rval(def, format, data);
+        rval.rowPitch = rowPitch;
+        rval.slicePitch = slicePitch;
+
+        if(resetOrigin)
+        {
+            if(PixelUtil::isCompressed(format))
+            {
+                if(rval.front > 0)
+                {
+                    rval.data = (uint8*)rval.data + rval.front * PixelUtil::getMemorySize(getWidth(), getHeight(), 1, format);
+                    rval.back -= rval.front;
+                    rval.front = 0;
+                }
+            }
+            else
+            {
+                rval.data = rval.getTopLeftFrontPixelPtr();
+                rval.right -= rval.left;
+                rval.bottom -= rval.top;
+                rval.back -= rval.front;
+                rval.front = rval.top = rval.left = 0;
+            }
+        }
+
+        return rval;
+    }
+    uchar* PixelBox::getTopLeftFrontPixelPtr() const
+    {
+        return data + (left + top * rowPitch + front * slicePitch) * PixelUtil::getNumElemBytes(format);
     }
     //-----------------------------------------------------------------------
     /**
@@ -95,32 +97,36 @@ namespace Ogre {
     {
         return getDescriptionFor(format).elemBytes;
     }
-	//-----------------------------------------------------------------------
-	size_t PixelUtil::getMemorySize(uint32 width, uint32 height, uint32 depth, PixelFormat format)
-	{
-		if(isCompressed(format))
-		{
-			switch(format)
-			{
-				// DXT formats work by dividing the image into 4x4 blocks, then encoding each
-				// 4x4 block with a certain number of bytes. 
-				case PF_DXT1:
-					return ((width+3)/4)*((height+3)/4)*8 * depth;
-				case PF_DXT2:
-				case PF_DXT3:
-				case PF_DXT4:
-				case PF_DXT5:
-					return ((width+3)/4)*((height+3)/4)*16 * depth;
+    //-----------------------------------------------------------------------
+    static size_t astc_slice_size(uint32 width, uint32 height, uint32 blockWidth, uint32 blockHeight)
+    {
+        return ((width + blockWidth - 1) / blockWidth) *
+               ((height + blockHeight - 1) / blockHeight) * 16;
+    }
+    size_t PixelUtil::getMemorySize(uint32 width, uint32 height, uint32 depth, PixelFormat format)
+    {
+        if(isCompressed(format))
+        {
+            switch(format)
+            {
+                // DXT formats work by dividing the image into 4x4 blocks, then encoding each
+                // 4x4 block with a certain number of bytes. 
+                case PF_DXT1:
+                    return ((width+3)/4)*((height+3)/4)*8 * depth;
+                case PF_DXT2:
+                case PF_DXT3:
+                case PF_DXT4:
+                case PF_DXT5:
+                    return ((width+3)/4)*((height+3)/4)*16 * depth;
                 case PF_BC4_SNORM:
                 case PF_BC4_UNORM:
-                    return ceilf(width/4.0f)*ceilf(height/4.0f)*8.0f;
+                    return ((width+3)/4)*((height+3)/4)*8 * depth;
                 case PF_BC5_SNORM:
                 case PF_BC5_UNORM:
                 case PF_BC6H_SF16:
                 case PF_BC6H_UF16:
                 case PF_BC7_UNORM:
-                case PF_BC7_UNORM_SRGB:
-                    return ceilf(width/4.0f)*ceilf(height/4.0f)*16.0f;
+                    return ((width+3)/4)*((height+3)/4)*16 * depth;
 
                 // Size calculations from the PVRTC OpenGL extension spec
                 // http://www.khronos.org/registry/gles/extensions/IMG/IMG_texture_compression_pvrtc.txt
@@ -134,27 +140,58 @@ namespace Ogre {
                 case PF_PVRTC2_4BPP:
                     return (std::max((int)width, 8) * std::max((int)height, 8) * 4 + 7) / 8;
 
+                // Size calculations from the ETC spec
+                // https://www.khronos.org/registry/OpenGL/extensions/OES/OES_compressed_ETC1_RGB8_texture.txt
                 case PF_ETC1_RGB8:
                 case PF_ETC2_RGB8:
                 case PF_ETC2_RGBA8:
                 case PF_ETC2_RGB8A1:
-                    return ((width * height) >> 1);
+                    return ((width + 3) / 4) * ((height + 3) / 4) * 8;
+
                 case PF_ATC_RGB:
                     return ((width + 3) / 4) * ((height + 3) / 4) * 8;
                 case PF_ATC_RGBA_EXPLICIT_ALPHA:
                 case PF_ATC_RGBA_INTERPOLATED_ALPHA:
                     return ((width + 3) / 4) * ((height + 3) / 4) * 16;
 
-				default:
-				OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid compressed pixel format",
-					"PixelUtil::getMemorySize");
-			}
-		}
-		else
-		{
-			return width*height*depth*getNumElemBytes(format);
-		}
-	}
+                case PF_ASTC_RGBA_4X4_LDR:
+                    return astc_slice_size(width, height, 4, 4) * depth;
+                case PF_ASTC_RGBA_5X4_LDR:
+                    return astc_slice_size(width, height, 5, 4) * depth;
+                case PF_ASTC_RGBA_5X5_LDR:
+                    return astc_slice_size(width, height, 5, 5) * depth;
+                case PF_ASTC_RGBA_6X5_LDR:
+                    return astc_slice_size(width, height, 6, 5) * depth;
+                case PF_ASTC_RGBA_6X6_LDR:
+                    return astc_slice_size(width, height, 6, 6) * depth;
+                case PF_ASTC_RGBA_8X5_LDR:
+                    return astc_slice_size(width, height, 8, 5) * depth;
+                case PF_ASTC_RGBA_8X6_LDR:
+                    return astc_slice_size(width, height, 8, 6) * depth;
+                case PF_ASTC_RGBA_8X8_LDR:
+                    return astc_slice_size(width, height, 8, 8) * depth;
+                case PF_ASTC_RGBA_10X5_LDR:
+                    return astc_slice_size(width, height, 10, 5) * depth;
+                case PF_ASTC_RGBA_10X6_LDR:
+                    return astc_slice_size(width, height, 10, 6) * depth;
+                case PF_ASTC_RGBA_10X8_LDR:
+                    return astc_slice_size(width, height, 10, 8) * depth;
+                case PF_ASTC_RGBA_10X10_LDR:
+                    return astc_slice_size(width, height, 10, 10) * depth;
+                case PF_ASTC_RGBA_12X10_LDR:
+                    return astc_slice_size(width, height, 12, 10) * depth;
+                case PF_ASTC_RGBA_12X12_LDR:
+                    return astc_slice_size(width, height, 12, 12) * depth;
+                default:
+                OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid compressed pixel format",
+                    "PixelUtil::getMemorySize");
+            }
+        }
+        else
+        {
+            return width*height*depth*getNumElemBytes(format);
+        }
+    }
     //-----------------------------------------------------------------------
     size_t PixelUtil::getNumElemBits( PixelFormat format )
     {
@@ -201,36 +238,6 @@ namespace Ogre {
         return (PixelUtil::getFlags(format) & PFF_LUMINANCE) > 0;
     }
     //-----------------------------------------------------------------------
-	bool PixelUtil::isValidExtent(size_t width, size_t height, size_t depth, PixelFormat format)
-	{
-		if(isCompressed(format))
-		{
-			switch(format)
-			{
-				case PF_DXT1:
-				case PF_DXT2:
-				case PF_DXT3:
-				case PF_DXT4:
-				case PF_DXT5:
-                case PF_BC4_SNORM:
-                case PF_BC4_UNORM:
-                case PF_BC5_SNORM:
-                case PF_BC5_UNORM:
-                case PF_BC6H_SF16:
-                case PF_BC6H_UF16:
-                case PF_BC7_UNORM:
-                case PF_BC7_UNORM_SRGB:
-					return ((width&3)==0 && (height&3)==0 && depth==1);
-				default:
-					return true;
-			}
-		}
-		else
-		{
-			return true;
-		}
-	}
-	//-----------------------------------------------------------------------
     void PixelUtil::getBitDepths(PixelFormat format, int rgba[4])
     {
         const PixelFormatDescription &des = getDescriptionFor(format);
@@ -239,8 +246,8 @@ namespace Ogre {
         rgba[2] = des.bbits;
         rgba[3] = des.abits;
     }
-	//-----------------------------------------------------------------------
-	void PixelUtil::getBitMasks(PixelFormat format, uint64 rgba[4])
+    //-----------------------------------------------------------------------
+    void PixelUtil::getBitMasks(PixelFormat format, uint64 rgba[4])
     {
         const PixelFormatDescription &des = getDescriptionFor(format);
         rgba[0] = des.rmask;
@@ -248,27 +255,24 @@ namespace Ogre {
         rgba[2] = des.bmask;
         rgba[3] = des.amask;
     }
-	//---------------------------------------------------------------------
-	void PixelUtil::getBitShifts(PixelFormat format, unsigned char rgba[4])
-	{
-		const PixelFormatDescription &des = getDescriptionFor(format);
-		rgba[0] = des.rshift;
-		rgba[1] = des.gshift;
-		rgba[2] = des.bshift;
-		rgba[3] = des.ashift;
-	}
+    //---------------------------------------------------------------------
+    void PixelUtil::getBitShifts(PixelFormat format, unsigned char rgba[4])
+    {
+        const PixelFormatDescription &des = getDescriptionFor(format);
+        rgba[0] = des.rshift;
+        rgba[1] = des.gshift;
+        rgba[2] = des.bshift;
+        rgba[3] = des.ashift;
+    }
     //-----------------------------------------------------------------------
-    String PixelUtil::getFormatName(PixelFormat srcformat)
+    const String& PixelUtil::getFormatName(PixelFormat srcformat)
     {
         return getDescriptionFor(srcformat).name;
     }
     //-----------------------------------------------------------------------
     bool PixelUtil::isAccessible(PixelFormat srcformat)
     {
-        if (srcformat == PF_UNKNOWN)
-            return false;
-        unsigned int flags = getFlags(srcformat);
-        return !((flags & PFF_COMPRESSED) || (flags & PFF_DEPTH));
+        return (srcformat != PF_UNKNOWN) && !isCompressed(srcformat);
     }
     //-----------------------------------------------------------------------
     PixelComponentType PixelUtil::getComponentType(PixelFormat fmt)
@@ -301,38 +305,20 @@ namespace Ogre {
                     return pf;
             }
         }
+
+        // allow look-up by alias name
+        if(tmp == "PF_BYTE_RGB")
+            return PF_BYTE_RGB;
+        if(tmp == "PF_BYTE_RGBA")
+            return PF_BYTE_RGBA;
+        if(tmp == "PF_BYTE_BGR")
+            return PF_BYTE_BGR;
+        if(tmp == "PF_BYTE_BGRA")
+            return PF_BYTE_BGRA;
+
         return PF_UNKNOWN;
     }
     //-----------------------------------------------------------------------
-    String PixelUtil::getBNFExpressionOfPixelFormats(bool accessibleOnly)
-    {
-        // Collect format names sorted by length, it's required by BNF compiler
-        // that similar tokens need longer ones comes first.
-        typedef multimap<String::size_type, String>::type FormatNameMap;
-        FormatNameMap formatNames;
-        for (size_t i = 0; i < PF_COUNT; ++i)
-        {
-            PixelFormat pf = static_cast<PixelFormat>(i);
-            if (!accessibleOnly || isAccessible(pf))
-            {
-                String formatName = getFormatName(pf);
-                formatNames.insert(std::make_pair(formatName.length(), formatName));
-            }
-        }
-
-        // Populate the BNF expression in reverse order
-        String result;
-        // Note: Stupid M$ VC7.1 can't dealing operator!= with FormatNameMap::const_reverse_iterator.
-        for (FormatNameMap::reverse_iterator j = formatNames.rbegin(); j != formatNames.rend(); ++j)
-        {
-            if (!result.empty())
-                result += " | ";
-            result += "'" + j->second + "'";
-        }
-
-        return result;
-    }
-    //-----------------------------------------------------------------------
     PixelFormat PixelUtil::getFormatForBitDepths(PixelFormat fmt, ushort integerBits, ushort floatBits)
     {
         switch (integerBits)
@@ -480,10 +466,10 @@ namespace Ogre {
             case PF_FLOAT32_R:
                 ((float*)dest)[0] = r;
                 break;
-			case PF_FLOAT32_GR:
-				((float*)dest)[0] = g;
-				((float*)dest)[1] = r;
-				break;
+            case PF_FLOAT32_GR:
+                ((float*)dest)[0] = g;
+                ((float*)dest)[1] = r;
+                break;
             case PF_FLOAT32_RGB:
                 ((float*)dest)[0] = r;
                 ((float*)dest)[1] = g;
@@ -495,14 +481,14 @@ namespace Ogre {
                 ((float*)dest)[2] = b;
                 ((float*)dest)[3] = a;
                 break;
-            case PF_DEPTH:
+            case PF_DEPTH16:
             case PF_FLOAT16_R:
                 ((uint16*)dest)[0] = Bitwise::floatToHalf(r);
                 break;
-			case PF_FLOAT16_GR:
-				((uint16*)dest)[0] = Bitwise::floatToHalf(g);
-				((uint16*)dest)[1] = Bitwise::floatToHalf(r);
-				break;
+            case PF_FLOAT16_GR:
+                ((uint16*)dest)[0] = Bitwise::floatToHalf(g);
+                ((uint16*)dest)[1] = Bitwise::floatToHalf(r);
+                break;
             case PF_FLOAT16_RGB:
                 ((uint16*)dest)[0] = Bitwise::floatToHalf(r);
                 ((uint16*)dest)[1] = Bitwise::floatToHalf(g);
@@ -515,20 +501,30 @@ namespace Ogre {
                 ((uint16*)dest)[3] = Bitwise::floatToHalf(a);
                 break;
             case PF_SHORT_RGB:
-				((uint16*)dest)[0] = (uint16)Bitwise::floatToFixed(r, 16);
+                ((uint16*)dest)[0] = (uint16)Bitwise::floatToFixed(r, 16);
                 ((uint16*)dest)[1] = (uint16)Bitwise::floatToFixed(g, 16);
                 ((uint16*)dest)[2] = (uint16)Bitwise::floatToFixed(b, 16);
                 break;
-			case PF_SHORT_RGBA:
-				((uint16*)dest)[0] = (uint16)Bitwise::floatToFixed(r, 16);
+            case PF_SHORT_RGBA:
+                ((uint16*)dest)[0] = (uint16)Bitwise::floatToFixed(r, 16);
                 ((uint16*)dest)[1] = (uint16)Bitwise::floatToFixed(g, 16);
                 ((uint16*)dest)[2] = (uint16)Bitwise::floatToFixed(b, 16);
                 ((uint16*)dest)[3] = (uint16)Bitwise::floatToFixed(a, 16);
-				break;
-			case PF_BYTE_LA:
-				((uint8*)dest)[0] = (uint8)Bitwise::floatToFixed(r, 8);
+                break;
+            case PF_BYTE_LA:
+                ((uint8*)dest)[0] = (uint8)Bitwise::floatToFixed(r, 8);
                 ((uint8*)dest)[1] = (uint8)Bitwise::floatToFixed(a, 8);
-				break;
+                break;
+            case PF_A2B10G10R10:
+            {
+                const uint16 ir = static_cast<uint16>( Math::saturate( r ) * 1023.0f + 0.5f );
+                const uint16 ig = static_cast<uint16>( Math::saturate( g ) * 1023.0f + 0.5f );
+                const uint16 ib = static_cast<uint16>( Math::saturate( b ) * 1023.0f + 0.5f );
+                const uint16 ia = static_cast<uint16>( Math::saturate( a ) * 3.0f + 0.5f );
+
+                ((uint32*)dest)[0] = (ia << 30u) | (ir << 20u) | (ig << 10u) | (ib);
+                break;
+            }
             default:
                 // Not yet supported
                 OGRE_EXCEPT(
@@ -616,11 +612,11 @@ namespace Ogre {
                 *r = *g = *b = ((const float*)src)[0];
                 *a = 1.0f;
                 break;
-			case PF_FLOAT32_GR:
-				*g = ((const float*)src)[0];
-				*r = *b = ((const float*)src)[1];
-				*a = 1.0f;
-				break;
+            case PF_FLOAT32_GR:
+                *g = ((const float*)src)[0];
+                *r = *b = ((const float*)src)[1];
+                *a = 1.0f;
+                break;
             case PF_FLOAT32_RGB:
                 *r = ((const float*)src)[0];
                 *g = ((const float*)src)[1];
@@ -633,7 +629,6 @@ namespace Ogre {
                 *b = ((const float*)src)[2];
                 *a = ((const float*)src)[3];
                 break;
-            case PF_DEPTH:
             case PF_FLOAT16_R:
                 *r = *g = *b = Bitwise::halfToFloat(((const uint16*)src)[0]);
                 *a = 1.0f;
@@ -686,7 +681,7 @@ namespace Ogre {
         void *destp, PixelFormat dstFormat, unsigned int count)
     {
         PixelBox src(count, 1, 1, srcFormat, srcp),
-				 dst(count, 1, 1, dstFormat, destp);
+                 dst(count, 1, 1, dstFormat, destp);
 
         bulkPixelConversion(src, dst);
     }
@@ -694,40 +689,44 @@ namespace Ogre {
     void PixelUtil::bulkPixelConversion(const PixelBox &src, const PixelBox &dst)
     {
         assert(src.getWidth() == dst.getWidth() &&
-			   src.getHeight() == dst.getHeight() &&
-			   src.getDepth() == dst.getDepth());
+               src.getHeight() == dst.getHeight() &&
+               src.getDepth() == dst.getDepth());
 
-		// Check for compressed formats, we don't support decompression, compression or recoding
-		if(PixelUtil::isCompressed(src.format) || PixelUtil::isCompressed(dst.format))
-		{
-			if(src.format == dst.format)
-			{
-				memcpy(dst.data, src.data, src.getConsecutiveSize());
-				return;
-			}
-			else
-			{
-				OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED,
-					"This method can not be used to compress or decompress images",
-					"PixelUtil::bulkPixelConversion");
-			}
-		}
+        // Check for compressed formats, we don't support decompression, compression or recoding
+        if(PixelUtil::isCompressed(src.format) || PixelUtil::isCompressed(dst.format))
+        {
+            if(src.format == dst.format && src.isConsecutive() && dst.isConsecutive())
+            {
+                // we can copy with slice granularity, useful for Tex2DArray handling
+                size_t bytesPerSlice = getMemorySize(src.getWidth(), src.getHeight(), 1, src.format);
+                memcpy(dst.data + bytesPerSlice * dst.front,
+                    src.data + bytesPerSlice * src.front,
+                    bytesPerSlice * src.getDepth());
+                return;
+            }
+            else
+            {
+                OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED,
+                    "This method can not be used to compress or decompress images",
+                    "PixelUtil::bulkPixelConversion");
+            }
+        }
 
         // The easy case
         if(src.format == dst.format) {
             // Everything consecutive?
             if(src.isConsecutive() && dst.isConsecutive())
             {
-				memcpy(dst.data, src.data, src.getConsecutiveSize());
+                memcpy(dst.getTopLeftFrontPixelPtr(), src.getTopLeftFrontPixelPtr(), src.getConsecutiveSize());
                 return;
             }
 
             const size_t srcPixelSize = PixelUtil::getNumElemBytes(src.format);
             const size_t dstPixelSize = PixelUtil::getNumElemBytes(dst.format);
-            uint8 *srcptr = static_cast<uint8*>(src.data)
+            uint8 *srcptr = src.data
                 + (src.left + src.top * src.rowPitch + src.front * src.slicePitch) * srcPixelSize;
-            uint8 *dstptr = static_cast<uint8*>(dst.data)
-				+ (dst.left + dst.top * dst.rowPitch + dst.front * dst.slicePitch) * dstPixelSize;
+            uint8 *dstptr = dst.data
+                + (dst.left + dst.top * dst.rowPitch + dst.front * dst.slicePitch) * dstPixelSize;
 
             // Calculate pitches+skips in bytes
             const size_t srcRowPitchBytes = src.rowPitch*srcPixelSize;
@@ -744,7 +743,7 @@ namespace Ogre {
             {
                 for(size_t y=src.top; y<src.bottom; y++)
                 {
-					memcpy(dstptr, srcptr, rowSize);
+                    memcpy(dstptr, srcptr, rowSize);
                     srcptr += srcRowPitchBytes;
                     dstptr += dstRowPitchBytes;
                 }
@@ -753,28 +752,28 @@ namespace Ogre {
             }
             return;
         }
-		// Converting to PF_X8R8G8B8 is exactly the same as converting to
-		// PF_A8R8G8B8. (same with PF_X8B8G8R8 and PF_A8B8G8R8)
-		if(dst.format == PF_X8R8G8B8 || dst.format == PF_X8B8G8R8)
-		{
-			// Do the same conversion, with PF_A8R8G8B8, which has a lot of
-			// optimized conversions
-			PixelBox tempdst = dst;
-			tempdst.format = dst.format==PF_X8R8G8B8?PF_A8R8G8B8:PF_A8B8G8R8;
-			bulkPixelConversion(src, tempdst);
-			return;
-		}
-		// Converting from PF_X8R8G8B8 is exactly the same as converting from
-		// PF_A8R8G8B8, given that the destination format does not have alpha.
-		if((src.format == PF_X8R8G8B8||src.format == PF_X8B8G8R8) && !hasAlpha(dst.format))
-		{
-			// Do the same conversion, with PF_A8R8G8B8, which has a lot of
-			// optimized conversions
-			PixelBox tempsrc = src;
-			tempsrc.format = src.format==PF_X8R8G8B8?PF_A8R8G8B8:PF_A8B8G8R8;
-			bulkPixelConversion(tempsrc, dst);
-			return;
-		}
+        // Converting to PF_X8R8G8B8 is exactly the same as converting to
+        // PF_A8R8G8B8. (same with PF_X8B8G8R8 and PF_A8B8G8R8)
+        if(dst.format == PF_X8R8G8B8 || dst.format == PF_X8B8G8R8)
+        {
+            // Do the same conversion, with PF_A8R8G8B8, which has a lot of
+            // optimized conversions
+            PixelBox tempdst = dst;
+            tempdst.format = dst.format==PF_X8R8G8B8?PF_A8R8G8B8:PF_A8B8G8R8;
+            bulkPixelConversion(src, tempdst);
+            return;
+        }
+        // Converting from PF_X8R8G8B8 is exactly the same as converting from
+        // PF_A8R8G8B8, given that the destination format does not have alpha.
+        if((src.format == PF_X8R8G8B8||src.format == PF_X8B8G8R8) && !hasAlpha(dst.format))
+        {
+            // Do the same conversion, with PF_A8R8G8B8, which has a lot of
+            // optimized conversions
+            PixelBox tempsrc = src;
+            tempsrc.format = src.format==PF_X8R8G8B8?PF_A8R8G8B8:PF_A8B8G8R8;
+            bulkPixelConversion(tempsrc, dst);
+            return;
+        }
 
 // NB VC6 can't handle the templates required for optimised conversion, tough
 #if OGRE_COMPILER != OGRE_COMPILER_MSVC || OGRE_COMP_VER >= 1300
@@ -788,13 +787,13 @@ namespace Ogre {
 
         const size_t srcPixelSize = PixelUtil::getNumElemBytes(src.format);
         const size_t dstPixelSize = PixelUtil::getNumElemBytes(dst.format);
-        uint8 *srcptr = static_cast<uint8*>(src.data)
+        uint8 *srcptr = src.data
             + (src.left + src.top * src.rowPitch + src.front * src.slicePitch) * srcPixelSize;
-        uint8 *dstptr = static_cast<uint8*>(dst.data)
+        uint8 *dstptr = dst.data
             + (dst.left + dst.top * dst.rowPitch + dst.front * dst.slicePitch) * dstPixelSize;
-		
-		// Old way, not taking into account box dimensions
-		//uint8 *srcptr = static_cast<uint8*>(src.data), *dstptr = static_cast<uint8*>(dst.data);
+        
+        // Old way, not taking into account box dimensions
+        //uint8 *srcptr = static_cast<uint8*>(src.data), *dstptr = static_cast<uint8*>(dst.data);
 
         // Calculate pitches+skips in bytes
         const size_t srcRowSkipBytes = src.getRowSkip()*srcPixelSize;
@@ -825,13 +824,13 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void PixelUtil::bulkPixelVerticalFlip(const PixelBox &box)
     {
-		// Check for compressed formats, we don't support decompression, compression or recoding
-		if(PixelUtil::isCompressed(box.format))
-		{
+        // Check for compressed formats, we don't support decompression, compression or recoding
+        if(PixelUtil::isCompressed(box.format))
+        {
             OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED,
                         "This method can not be used for compressed formats",
                         "PixelUtil::bulkPixelVerticalFlip");
-		}
+        }
         
         const size_t pixelSize = PixelUtil::getNumElemBytes(box.format);
         const size_t copySize = (box.right - box.left) * pixelSize;
@@ -840,10 +839,10 @@ namespace Ogre {
         const size_t rowPitchBytes = box.rowPitch * pixelSize;
         const size_t slicePitchBytes = box.slicePitch * pixelSize;
 
-        uint8 *basesrcptr = static_cast<uint8*>(box.data)
+        uint8 *basesrcptr = box.data
             + (box.left + box.top * box.rowPitch + box.front * box.slicePitch) * pixelSize;
         uint8 *basedstptr = basesrcptr + (box.bottom - box.top - 1) * rowPitchBytes;
-        uint8* tmpptr = (uint8*)OGRE_MALLOC_ALIGN(copySize, MEMCATEGORY_GENERAL, false);
+        uint8* tmpptr = (uint8*)OGRE_MALLOC_SIMD(copySize, MEMCATEGORY_GENERAL);
         
         // swap rows
         const size_t halfRowCount = (box.bottom - box.top) >> 1;
@@ -864,14 +863,14 @@ namespace Ogre {
             basedstptr += slicePitchBytes;
         }
         
-        OGRE_FREE_ALIGN(tmpptr, MEMCATEGORY_GENERAL, false);
+        OGRE_FREE_SIMD(tmpptr, MEMCATEGORY_GENERAL);
     }
 
-    ColourValue PixelBox::getColourAt(size_t x, size_t y, size_t z)
+    ColourValue PixelBox::getColourAt(size_t x, size_t y, size_t z) const
     {
         ColourValue cv;
 
-        unsigned char pixelSize = PixelUtil::getNumElemBytes(format);
+        size_t pixelSize = PixelUtil::getNumElemBytes(format);
         size_t pixelOffset = pixelSize * (z * slicePitch + y * rowPitch + x);
         PixelUtil::unpackColour(&cv, format, (unsigned char *)data + pixelOffset);
 
@@ -880,7 +879,7 @@ namespace Ogre {
 
     void PixelBox::setColourAt(ColourValue const &cv, size_t x, size_t y, size_t z)
     {
-        unsigned char pixelSize = PixelUtil::getNumElemBytes(format);
+        size_t pixelSize = PixelUtil::getNumElemBytes(format);
         size_t pixelOffset = pixelSize * (z * slicePitch + y * rowPitch + x);
         PixelUtil::packColour(cv, format, (unsigned char *)data + pixelOffset);
     }
