@@ -145,11 +145,7 @@ namespace Ogre {
 
         // Upfront, lets check whether we have vertex program capability
         RenderSystem* rend = Root::getSingleton().getRenderSystem();
-        bool useVertexPrograms = false;
-        if (rend && rend->getCapabilities()->hasCapability(RSC_VERTEX_PROGRAM))
-        {
-            useVertexPrograms = true;
-        }
+        bool useVertexPrograms = rend;
 
 
         // Look for a position element
