@@ -60,7 +60,7 @@ namespace Ogre {
         could also be used. The Quake3 level load process is in a different
         class called Quake3Level to keep the specifics separate.
     */
-    class BspLevel : public Resource
+    class BspLevel : public Resource, public Renderable
     {
         friend class BspSceneManager;
 
@@ -109,8 +109,40 @@ namespace Ogre {
         /** Get sky curvature */
         Real getSkyCurvature(void) const;
 
+        void getRenderOperation(RenderOperation &op) override
+        {
+            op = mRenderOp;
+        }
+
+        void getWorldTransforms(Matrix4* xform) const override
+        {
+            *xform = Matrix4::IDENTITY;
+        }
+
+        Real getSquaredViewDepth(const Camera *cam) const override
+        {
+            // always visible
+            return -1;
+        }
+
+        const LightList& getLights() const override
+        {
+            static LightList lights;
+            return lights;
+        }
+
+        const MaterialPtr& getMaterial(void) const override
+        {
+            static MaterialPtr nullPtr;
+            return nullPtr;
+        }
+
         /** Utility class just to enable queueing of patches */
     protected:
+        /** Caches a face group for imminent rendering. */
+        uint32 cacheGeometry(uint32* pIndexes, const StaticFaceGroup* faceGroup);
+        bool cacheGeometry(const std::vector<StaticFaceGroup*>& materialFaceGroup);
+
         /** @copydoc Resource::loadImpl. */
         void loadImpl(void);
         /** @copydoc Resource::unloadImpl. */
@@ -146,7 +178,7 @@ namespace Ogre {
         };
 
         /// Vertex data holding all the data for the level, but able to render parts of it
-        VertexData* mVertexData;
+        RenderOperation mRenderOp;
 
         /** Array of indexes into the mFaceGroups array. This buffer is organised
             by leaf node so leaves can just use contiguous chunks of it and
@@ -159,8 +191,6 @@ namespace Ogre {
         int mNumFaceGroups;
 
         /// Indexes for the whole level, will be copied to the real indexdata per frame
-        size_t mNumIndexes;
-        /// System-memory buffer
         HardwareIndexBufferSharedPtr mIndexes;
 
         /// Brushes as used for collision, main memory is here
