@@ -337,7 +337,6 @@ namespace Ogre {
         GpuConstantDefinitionMap::const_iterator iterE = mParametersMap.end();
         for (; iter != iterE ; ++iter)
         {
-            const String & paramName = iter->first;
             GpuConstantDefinition def = iter->second;
 
             mConstantDefs->map.emplace(iter->first, iter->second);
@@ -357,9 +356,6 @@ namespace Ogre {
                         GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL));
                 mIntLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
             }
-
-            // Deal with array indexing
-            mConstantDefs->generateConstantDefinitionArrayEntries(paramName, def);
         }
         
     }
@@ -581,6 +577,7 @@ namespace Ogre {
         ResourceHandle handle, const String& group, bool isManual, 
         ManualResourceLoader* loader)
         : HighLevelGpuProgram(creator, name, handle, group, isManual, loader)
+        , mEntryPoint("main")
         , mColumnMajorMatrices(true)
         , mBackwardsCompatibility(false)
         , mMicroCode(NULL)
@@ -652,24 +649,20 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void D3D9HLSLProgram::setTarget(const String& target)
     {
-        mTarget = "";
         std::vector<String> profiles = StringUtil::split(target, " ");
 
+        // find first supported format
         for(unsigned int i = 0 ; i < profiles.size() ; i++)
         {
             String & currentProfile = profiles[i];
             if(GpuProgramManager::getSingleton().isSyntaxSupported(currentProfile))
             {
                 mTarget = currentProfile;
-                break;
+                return;
             }
         }
 
-        if(mTarget == "")
-        {
-            LogManager::getSingleton().logMessage(
-                "Invalid target for D3D9 shader '" + mName + "' - '" + target + "'");
-        }
+        mTarget = profiles.front();
     }
 
     //-----------------------------------------------------------------------
