@@ -35,9 +35,7 @@ Copyright (c) 2000-2014 Torus Knot Software Ltd
 #include "OgreLight.h"
 #include "OgreCamera.h"
 #include "OgreGL3PlusTextureManager.h"
-#include "OgreGL3PlusHardwareCounterBuffer.h"
 #include "OgreGL3PlusHardwareUniformBuffer.h"
-#include "OgreGL3PlusHardwareShaderStorageBuffer.h"
 #include "OgreGL3PlusHardwareVertexBuffer.h"
 #include "OgreGL3PlusHardwareIndexBuffer.h"
 #include "OgreGLSLShader.h"
@@ -168,6 +166,7 @@ namespace Ogre {
         mCurrentShader.fill(NULL);
         mLargestSupportedAnisotropy = 1;
         mRTTManager = NULL;
+        mSeparateShaderObjectsEnabled = false;
     }
 
     GL3PlusRenderSystem::~GL3PlusRenderSystem()
@@ -201,6 +200,13 @@ namespace Ogre {
         opt.immutable = false;
 
         mOptions[opt.name] = opt;
+
+        opt.name = "Separate Shader Objects";
+        opt.possibleValues = {"No", "Yes"};
+        opt.currentValue = opt.possibleValues[0];
+        opt.immutable = false;
+
+        mOptions[opt.name] = opt;
     }
 
     RenderSystemCapabilities* GL3PlusRenderSystem::createRenderSystemCapabilities() const
@@ -314,9 +320,9 @@ namespace Ogre {
         // Check for non-power-of-2 texture support
         rsc->setCapability(RSC_NON_POWER_OF_2_TEXTURES);
 
-        // Check for atomic counter support
-        if (hasMinGLVersion(4, 2) || checkExtension("GL_ARB_shader_atomic_counters"))
-            rsc->setCapability(RSC_ATOMIC_COUNTERS);
+        // Check for SSBO support
+        if (hasMinGLVersion(4, 3) || checkExtension("GL_ARB_shader_storage_buffer_object"))
+            rsc->setCapability(RSC_READ_WRITE_BUFFERS);
 
         // Scissor test is standard
         rsc->setCapability(RSC_SCISSOR_TEST);
@@ -361,16 +367,17 @@ namespace Ogre {
         if (getNativeShadingLanguageVersion() >= 130 && !limitedOSXCoreProfile)
             rsc->addShaderProfile("glsl130");
 
-        if(checkExtension("GL_ARB_gl_spirv")) rsc->addShaderProfile("spirv");
-
-        if (hasMinGLVersion(4, 1) || checkExtension("GL_ARB_separate_shader_objects")) {
-            // this relaxes shader matching rules and requires slightly different GLSL declaration
-            // however our usage pattern does not benefit from this and driver support is quite poor
-            // so disable it for now (see below)
-            /*rsc->setCapability(RSC_SEPARATE_SHADER_OBJECTS);
-            rsc->setCapability(RSC_GLSL_SSO_REDECLARE);*/
+        if (mSeparateShaderObjectsEnabled &&
+            (hasMinGLVersion(4, 3) ||
+             (checkExtension("GL_ARB_separate_shader_objects") && checkExtension("GL_ARB_program_interface_query"))))
+        {
+            rsc->setCapability(RSC_SEPARATE_SHADER_OBJECTS);
+            rsc->setCapability(RSC_GLSL_SSO_REDECLARE);
         }
 
+        if (checkExtension("GL_ARB_gl_spirv") && rsc->hasCapability(RSC_SEPARATE_SHADER_OBJECTS))
+            rsc->addShaderProfile("spirv");
+
         // Mesa 11.2 does not behave according to spec and throws a "gl_Position redefined"
         if(rsc->getDeviceName().find("Mesa") != String::npos) {
             rsc->unsetCapability(RSC_GLSL_SSO_REDECLARE);
@@ -667,6 +674,12 @@ namespace Ogre {
                 }
             }
 
+            it = mOptions.find("Separate Shader Objects");
+            if (it != mOptions.end())
+            {
+                mSeparateShaderObjectsEnabled = StringConverter::parseBool(it->second.currentValue);
+            }
+
             // Initialise GL after the first window has been created
             // TODO: fire this from emulation options, and don't duplicate Real and Current capabilities
             mRealCapabilities = createRenderSystemCapabilities();
@@ -2041,14 +2054,7 @@ namespace Ogre {
 
         if (mask & (uint16)GPV_GLOBAL)
         {
-            // TODO We could maybe use GL_EXT_bindable_uniform here to produce Dx10-style
-            // shared constant buffers, but GPU support seems fairly weak?
-            // check the match to constant buffers & use rendersystem data hooks to store
-            // for now, just copy
-            params->_copySharedParams();
-
-            program->updateUniformBlocks();
-            // program->updateShaderStorageBlock(params, mask, mType);
+            params->_updateSharedParams();
         }
 
         // Pass on parameters from params to program object uniforms.
