@@ -63,10 +63,6 @@ namespace Ogre {
             /// Create a uniform buffer
             HardwareUniformBufferSharedPtr createUniformBuffer(size_t sizeBytes, HardwareBuffer::Usage usage,
                                                                bool useShadowBuffer, size_t binding, const String& name = "");
-            HardwareCounterBufferSharedPtr createCounterBuffer(size_t sizeBytes,
-                                                               HardwareBuffer::Usage usage = HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY_DISCARDABLE,
-                                                               bool useShadowBuffer = false, const String& name = "");
-
             /// Utility function to get the correct GL type based on VET's
             static GLenum getGLType(VertexElementType type);
 
