@@ -64,9 +64,7 @@ namespace Ogre {
 
         PixelBox scaled;
 
-        if (src.getWidth() != dstBox.getWidth() ||
-            src.getHeight() != dstBox.getHeight() ||
-            src.getDepth() != dstBox.getDepth())
+        if (src.getSize() != dstBox.getSize())
         {
             // Scale to destination size.
             // This also does pixel format conversion if needed
@@ -101,12 +99,9 @@ namespace Ogre {
                         "GLES2HardwarePixelBuffer::blitToMemory");
         }
 
-        if (srcBox.left == 0 && srcBox.right == getWidth() &&
-            srcBox.top == 0 && srcBox.bottom == getHeight() &&
-            srcBox.front == 0 && srcBox.back == getDepth() &&
-            dst.getWidth() == getWidth() &&
-            dst.getHeight() == getHeight() &&
-            dst.getDepth() == getDepth() &&
+        if (srcBox.getOrigin() == Vector3i(0, 0 ,0) &&
+            srcBox.getSize() == getSize() &&
+            dst.getSize() == getSize() &&
             GLES2PixelUtil::getGLInternalFormat(dst.format) != 0)
         {
             // The direct case: the user wants the entire texture in a format supported by GL
@@ -119,9 +114,7 @@ namespace Ogre {
             allocateBuffer();
             // Download entire buffer
             download(mBuffer);
-            if(srcBox.getWidth() != dst.getWidth() ||
-                srcBox.getHeight() != dst.getHeight() ||
-                srcBox.getDepth() != dst.getDepth())
+            if(srcBox.getSize() != dst.getSize())
             {
                 // We need scaling
                 Image::scale(mBuffer.getSubVolume(srcBox), dst, Image::FILTER_BILINEAR);
@@ -363,9 +356,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------------  
     void GLES2TextureBuffer::download(const PixelBox &data)
     {
-        if(data.getWidth() != getWidth() ||
-           data.getHeight() != getHeight() ||
-           data.getDepth() != getDepth())
+        if(data.getSize() != getSize())
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "only download of entire buffer is supported by GL ES",
                         "GLES2TextureBuffer::download");
 
@@ -435,11 +426,8 @@ namespace Ogre {
     void GLES2TextureBuffer::blit(const HardwarePixelBufferSharedPtr &src, const Box &srcBox, const Box &dstBox)
     {
         GLES2TextureBuffer *srct = static_cast<GLES2TextureBuffer *>(src.get());
-        // TODO: blitFromTexture currently broken
-        // Destination texture must be 2D or Cube
-        // Source texture must be 2D
-        if (false) //(((src->getUsage() & TU_RENDERTARGET) == 0 && (srct->mTarget == GL_TEXTURE_2D)) ||
-            //((srct->mTarget == GL_TEXTURE_3D_OES) && (mTarget != GL_TEXTURE_2D_ARRAY)))
+        if ((srcBox.getWidth() == dstBox.getWidth() && srcBox.getHeight() == dstBox.getHeight() &&
+             srcBox.getDepth() == 1))
         {
             blitFromTexture(srct, srcBox, dstBox);
         }
@@ -450,219 +438,51 @@ namespace Ogre {
     }
     
     //-----------------------------------------------------------------------------  
-    // Very fast texture-to-texture blitter and hardware bi/trilinear scaling implementation using FBO
-    // Destination texture must be 1D, 2D, 3D, or Cube
-    // Source texture must be 1D, 2D or 3D
-    // Supports compressed formats as both source and destination format, it will use the hardware DXT compressor
-    // if available.
-    // @author W.J. van der Laan
     void GLES2TextureBuffer::blitFromTexture(GLES2TextureBuffer *src, const Box &srcBox, const Box &dstBox)
     {
-        OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED, "Not implemented",
-                    "GLES2TextureBuffer::blitFromTexture");
-        // todo - add a shader attach...
-//        std::cerr << "GLES2TextureBuffer::blitFromTexture " <<
-//        src->mTextureID << ":" << srcBox.left << "," << srcBox.top << "," << srcBox.right << "," << srcBox.bottom << " " <<
-//        mTextureID << ":" << dstBox.left << "," << dstBox.top << "," << dstBox.right << "," << dstBox.bottom << std::endl;
-
-        // Store reference to FBO manager
-        GLES2FBOManager *fboMan = static_cast<GLES2FBOManager *>(GLRTTManager::getSingletonPtr());
-
         GLES2RenderSystem* rs = getGLES2RenderSystem();
-        rs->_disableTextureUnitsFrom(0);
-        glActiveTexture(GL_TEXTURE0);
 
-        // Disable alpha, depth and scissor testing, disable blending,
-        // and disable culling
-        glDisable(GL_DEPTH_TEST);
-        glDisable(GL_SCISSOR_TEST);
-        glDisable(GL_BLEND);
-        glDisable(GL_CULL_FACE);
-
-        // Set up source texture
-        rs->_getStateCacheManager()->bindGLTexture(src->mTarget, src->mTextureID);
-
-        // Set filtering modes depending on the dimensions and source
-        if(srcBox.getWidth() == dstBox.getWidth() &&
-           srcBox.getHeight() == dstBox.getHeight() &&
-           srcBox.getDepth() == dstBox.getDepth())
-        {
-            // Dimensions match -- use nearest filtering (fastest and pixel correct)
-            if(src->mUsage & TU_AUTOMIPMAP)
-            {
-                OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST));
-            }
-            else
-            {
-                OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST));
-            }
-            OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_MAG_FILTER, GL_NEAREST));
-        }
-        else
-        {
-            // Dimensions don't match -- use bi or trilinear filtering depending on the
-            // source texture.
-            if(src->mUsage & TU_AUTOMIPMAP)
-            {
-                // Automatic mipmaps, we can safely use trilinear filter which
-                // brings greatly improved quality for minimisation.
-                OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR));
-            }
-            else
-            {
-                // Manual mipmaps, stay safe with bilinear filtering so that no
-                // intermipmap leakage occurs.
-                OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR));
-            }
-            OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR));
-        }
-        // Clamp to edge (fastest)
-        OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE));
-        OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE));
-
-        if(src->getDepth() > 1) {
-            OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_WRAP_R_OES, GL_CLAMP_TO_EDGE));
-        }
-
-#if OGRE_NO_GLES3_SUPPORT == 0
-        // Set origin base level mipmap to make sure we source from the right mip
-        // level.
-        OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_BASE_LEVEL, src->mLevel));
-#endif
         // Store old binding so it can be restored later
         GLint oldfb;
         OGRE_CHECK_GL_ERROR(glGetIntegerv(GL_FRAMEBUFFER_BINDING, &oldfb));
 
         // Set up temporary FBO
-        OGRE_CHECK_GL_ERROR(glBindFramebuffer(GL_FRAMEBUFFER, fboMan->getTemporaryFBO()));
+        GLuint tempFBO;
+        OGRE_CHECK_GL_ERROR(glGenFramebuffers(1, &tempFBO));
+        OGRE_CHECK_GL_ERROR(glBindFramebuffer(GL_FRAMEBUFFER, tempFBO));
 
-        TexturePtr tempTex;
-        if(!fboMan->checkFormat(mFormat))
+        src->bindToFramebuffer(GL_COLOR_ATTACHMENT0, 0);
+        rs->_getStateCacheManager()->bindGLTexture(mTarget, mTextureID);
+
+        switch (mTarget)
         {
-            // If target format not directly supported, create intermediate texture
-            tempTex = TextureManager::getSingleton().createManual(
-                "GLBlitFromTextureTMP", ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, TEX_TYPE_2D,
-                dstBox.getWidth(), dstBox.getHeight(), dstBox.getDepth(), 0,
-                fboMan->getSupportedAlternative(mFormat));
-
-            OGRE_CHECK_GL_ERROR(
-                glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D,
-                                       static_pointer_cast<GLES2Texture>(tempTex)->getGLID(), 0));
-            // Set viewport to size of destination slice
-            OGRE_CHECK_GL_ERROR(glViewport(0, 0, dstBox.getWidth(), dstBox.getHeight()));
-        }
-        else
-        {
-            // We are going to bind directly, so set viewport to size and position of destination slice
-            OGRE_CHECK_GL_ERROR(glViewport(dstBox.left, dstBox.top, dstBox.getWidth(), dstBox.getHeight()));
-        }
-
-        // Select scratch VAO #0
-        if(rs->getCapabilities()->hasCapability(RSC_VAO))
-            rs->_getStateCacheManager()->bindGLVertexArray(0);
-
-        // Process each destination slice
-        for(size_t slice = dstBox.front; slice < dstBox.back; ++slice)
-        {
-            if(!tempTex)
+        case GL_TEXTURE_2D:
+        case GL_TEXTURE_CUBE_MAP:
+            OGRE_CHECK_GL_ERROR(glCopyTexSubImage2D(mFaceTarget, mLevel, dstBox.left, dstBox.top,
+                                                    srcBox.left, srcBox.top, dstBox.getWidth(),
+                                                    dstBox.getHeight()));
+            break;
+        case GL_TEXTURE_3D:
+        case GL_TEXTURE_2D_ARRAY:
+            // Process each destination slice
+            for (uint32 slice = dstBox.front; slice < dstBox.back; ++slice)
             {
-                // Bind directly
-                bindToFramebuffer(GL_COLOR_ATTACHMENT0, slice);
-            }
-
-            // Calculate source texture coordinates
-            float u1 = (float)srcBox.left / (float)src->mWidth;
-            float v1 = (float)srcBox.top / (float)src->mHeight;
-            float u2 = (float)srcBox.right / (float)src->mWidth;
-            float v2 = (float)srcBox.bottom / (float)src->mHeight;
-            // Calculate source slice for this destination slice
-            float w = (float)(slice - dstBox.front) / (float)dstBox.getDepth();
-            // Get slice # in source
-            w = w * (float)srcBox.getDepth() + srcBox.front;
-            // Normalise to texture coordinate in 0.0 .. 1.0
-            w = (w+0.5f) / (float)src->mDepth;
-
-            // Finally we're ready to rumble
-            rs->_getStateCacheManager()->bindGLTexture(src->mTarget, src->mTextureID);
-            OGRE_CHECK_GL_ERROR(glEnable(src->mTarget));
-
-            GLfloat squareVertices[] = {
-               -1.0f, -1.0f,
-                1.0f, -1.0f,
-               -1.0f,  1.0f,
-                1.0f,  1.0f,
-            };
-            GLfloat texCoords[] = {
-                u1, v1, w,
-                u2, v1, w,
-                u2, v2, w,
-                u1, v2, w
-            };
-
-            GLuint posAttrIndex = GLSLProgramCommon::getFixedAttributeIndex(VES_POSITION, 0);
-            GLuint texAttrIndex = GLSLProgramCommon::getFixedAttributeIndex(VES_TEXTURE_COORDINATES, 0);
-
-            // Draw the textured quad
-            OGRE_CHECK_GL_ERROR(glVertexAttribPointer(posAttrIndex,
-                                  2,
-                                  GL_FLOAT,
-                                  0,
-                                  0,
-                                  squareVertices));
-            OGRE_CHECK_GL_ERROR(glEnableVertexAttribArray(posAttrIndex));
-            OGRE_CHECK_GL_ERROR(glVertexAttribPointer(texAttrIndex,
-                                  3,
-                                  GL_FLOAT,
-                                  0,
-                                  0,
-                                  texCoords));
-            OGRE_CHECK_GL_ERROR(glEnableVertexAttribArray(texAttrIndex));
-
-            OGRE_CHECK_GL_ERROR(glDrawArrays(GL_TRIANGLE_STRIP, 0, 4));
-
-            OGRE_CHECK_GL_ERROR(glDisable(src->mTarget));
-
-            if(tempTex)
-            {
-                // Copy temporary texture
-                rs->_getStateCacheManager()->bindGLTexture(mTarget, mTextureID);
-                switch(mTarget)
-                {
-                    case GL_TEXTURE_2D:
-                    case GL_TEXTURE_CUBE_MAP:
-                        OGRE_CHECK_GL_ERROR(glCopyTexSubImage2D(mFaceTarget, mLevel,
-                                            dstBox.left, dstBox.top,
-                                            0, 0, dstBox.getWidth(), dstBox.getHeight()));
-                        break;
-                }
+                OGRE_CHECK_GL_ERROR(glCopyTexSubImage3D(mFaceTarget, mLevel, dstBox.left, dstBox.top, slice,
+                                                        srcBox.left, srcBox.top, dstBox.getWidth(),
+                                                        dstBox.getHeight()));
             }
+            break;
         }
-        // Finish up
-        if(!tempTex)
-        {
-            // Generate mipmaps
-            if(mUsage & TU_AUTOMIPMAP)
-            {
-                rs->_getStateCacheManager()->bindGLTexture(mTarget, mTextureID);
-                OGRE_CHECK_GL_ERROR(glGenerateMipmap(mTarget));
-            }
-        }
-
-        // Reset source texture to sane state
-        rs->_getStateCacheManager()->bindGLTexture(src->mTarget, src->mTextureID);
-
-#if OGRE_NO_GLES3_SUPPORT == 0
-        OGRE_CHECK_GL_ERROR(glTexParameteri(src->mTarget, GL_TEXTURE_BASE_LEVEL, 0));
-#endif
-        // Detach texture from temporary framebuffer
-        OGRE_CHECK_GL_ERROR(glFramebufferRenderbuffer(
-            GL_FRAMEBUFFER, PixelUtil::isDepth(mFormat) ? GL_DEPTH_ATTACHMENT : GL_COLOR_ATTACHMENT0,
-            GL_RENDERBUFFER, 0));
 
         // Restore old framebuffer
         OGRE_CHECK_GL_ERROR(glBindFramebuffer(GL_FRAMEBUFFER, oldfb));
-        if(tempTex)
-            TextureManager::getSingleton().remove(tempTex);
+        OGRE_CHECK_GL_ERROR(glDeleteFramebuffers(1, &tempFBO));
+
+        // Generate mipmaps
+        if (mUsage & TU_AUTOMIPMAP)
+        {
+            OGRE_CHECK_GL_ERROR(glGenerateMipmap(mTarget));
+        }
     }
     //-----------------------------------------------------------------------------  
     // blitFromMemory doing hardware trilinear scaling
@@ -672,9 +492,7 @@ namespace Ogre {
         // the source dimensions match the destination ones, in which case no scaling is needed
         // FIXME: always uses software path, as blitFromTexture is not implemented
         if(true ||
-           (src.getWidth() == dstBox.getWidth() &&
-            src.getHeight() == dstBox.getHeight() &&
-            src.getDepth() == dstBox.getDepth()))
+           (src.getSize() == dstBox.getSize()))
         {
             GLES2HardwarePixelBuffer::blitFromMemory(src, dstBox);
             return;
@@ -691,8 +509,8 @@ namespace Ogre {
             src.getWidth(), src.getHeight(), src.getDepth(), MIP_UNLIMITED, src.format);
 
         // Upload data to 0,0,0 in temporary texture
-        Box tempTarget(0, 0, 0, tex->getWidth(), tex->getHeight(), tex->getDepth());
-        tex->getBuffer()->blitFromMemory(src, tempTarget);
+        Box tempTarget(src.getSize());
+        tex->getBuffer()->blitFromMemory(src);
 
         // Blit from texture
         blit(tex->getBuffer(), tempTarget, dstBox);
